@@ -18,16 +18,28 @@ class Category(ModelSQL, ModelView):
         super(Category, cls).__setup__()
         cls._order.insert(0, ('name', 'ASC'))
 
-        cls._constraints += [
-            ('check_recursion', 'recursive_categories'),
-            ]
-        cls._error_messages.update({
-                'recursive_categories': \
-                    'You can not create recursive categories!',
-                })
+    @classmethod
+    def validate(cls, categories):
+        super(Category, cls).validate(categories)
+        cls.check_recursion(categories, rec_name='name')
 
     def get_rec_name(self, name):
         if self.parent:
             return self.parent.get_rec_name(name) + ' / ' + self.name
         else:
             return self.name
+
+    @classmethod
+    def search_rec_name(cls, name, clause):
+        if isinstance(clause[2], basestring):
+            values = clause[2].split('/')
+            values.reverse()
+            domain = []
+            field = 'name'
+            for name in values:
+                domain.append((field, clause[1], name.strip()))
+                field = 'parent.' + field
+        else:
+            domain = [('name',) + tuple(clause[1:])]
+        ids = [w.id for w in cls.search(domain, order=[])]
+        return [('parent', 'child_of', ids)]
